import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { BettingService } from 'app/modules/polyglot/api/services/betting.service';
import {
  ListBettingHistoryResponse,
  ListBettingHistoryRequest,
} from 'app/modules/proto/c2se/api/betting_pb';
import { Pagination } from 'app/modules/proto/protobuf/pagination_pb';
import { environment } from 'environments/environment';
import { catchError, Observable, throwError } from 'rxjs';

import { Slot } from '../models/slot';
import { SlotPagination } from '../models/slot-pagination';
import { SlotService } from '../services/slot.service';

@Injectable({
  providedIn: 'root',
})
export class SlotResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _slotService: SlotService, private _router: Router) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<Slot | undefined> {
    return this._slotService.getSlotById(route.paramMap.get('id')).pipe(
      // Error here means the requested product is not available
      catchError((error) => {
        // Log the error
        console.error(error);

        // Get the parent url
        const parentUrl = state.url.split('/').slice(0, -1).join('/');

        // Navigate to there
        this._router.navigateByUrl(parentUrl);

        // Throw an error
        return throwError(error);
      })
    );
  }
}

@Injectable({
  providedIn: 'root',
})
export class SlotsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _slotService: SlotService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: SlotPagination;
    slots: Slot[];
  }> {
    return this._slotService.getSlots();
  }
}

@Injectable({
  providedIn: 'root',
})
export class ListSlotResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _bettingService: BettingService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ):
    | Observable<ListBettingHistoryResponse.Result>
    | Promise<ListBettingHistoryResponse.Result>
    | ListBettingHistoryResponse.Result {
    const url = state.url;
    const startIdx = url.lastIndexOf('/');
    const target = url.substring(startIdx + 1);

    const req = new ListBettingHistoryRequest();
    const search = new ListBettingHistoryRequest.Search();
    const pagination = new Pagination();

    let vendorIds: number[] = environment.constants.venderIds.slots;

    search.setVendorIdsList(vendorIds);
    pagination.setPage(1);
    pagination.setPageSize(10);

    req.setSearch(search);
    req.setPagination(pagination);

    return this._bettingService.listVendors(req);
  }
}
