import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  QueryList,
  ViewChild,
  ViewChildren,
  ViewEncapsulation,
} from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';

import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  delay,
  map,
  merge,
  Observable,
  of,
  startWith,
  Subject,
  switchMap,
  take,
  takeUntil,
  timer,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { Partner } from '../models/partner';
import { PartnerPagination } from '../models/partner-pagination';
import { PartnerService } from '../services/partner.service';
import { ActivatedRoute, Router } from '@angular/router';
import {
  RegistComposeComponent,
  RegistComposeResult,
} from '../compose/regist-compose.component';
import { CashComposeComponent } from '../compose/cash-compose.component';
import { CommissionComposeComponent } from '../compose/commission-compose.component';
import { CompComposeComponent } from '../compose/comp-compose.component';
import { CouponComposeComponent } from '../compose/coupon-compose.component';
import { MessageComposeComponent } from '../compose/message-compose.component';

import { MatDialog } from '@angular/material/dialog';
import { SiteService } from 'app/modules/polyglot/site/services/site.service';
import { Site } from 'app/modules/proto/models/site_pb';
import { MemberService } from 'app/modules/polyglot/member/services/member.service';
import { MemberModel, MemberState } from 'app/modules/proto/models/member_pb';
import {
  ListMembersRequest,
  ListMembersResponse,
} from 'app/modules/proto/c2se/member_pb';
import { MatCheckbox, MatCheckboxChange } from '@angular/material/checkbox';
import { ListMemberLevelsResponse } from 'app/modules/proto/c2se/member_level_pb';
import { MemberLevel } from 'app/modules/proto/models/member_level_pb';
import { Pagination } from 'app/modules/proto/protobuf/pagination_pb';
import { VendorService } from 'app/modules/polyglot/api/services/vendor.service';
import { environment } from 'environments/environment';

@Component({
  selector: 'partner-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .partner-grid {
        /*                     관리   요율  매장  아이디  등급 */
        grid-template-columns: 40px 40px 100px auto 40px;

        @screen sm {
          /*                     관리   요율  매장  아이디  등급 예금주 */
          grid-template-columns: 40px 40px 100px auto 40px 140px;
        }

        @screen md {
          /*                     관리   요율  매장  아이디  등급  예금주 정산 보유금 */
          grid-template-columns: 80px auto 80px 40px 40px 120px 120px 120px 120px 120px 100px;
        }

        @screen lg {
          /*                     관리  아이디 등급  예금주 정산  보유금   충전  환전금 수익금 가입   비고 */
          grid-template-columns: 60px auto 60px 40px 40px 100px 100px 100px 100px 120px 100px;
        }
      }
      .mat-tooltip {
        white-space: pre-line;
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  @ViewChildren('chkUsers') chkUsers!: QueryList<MatCheckbox>;

  listPartner$!: Observable<MemberModel[] | undefined>;
  listMemberLevels$!: Observable<MemberLevel[] | undefined>;

  __isLevelOpened = false;
  isLoading = false;
  searchInputControl = new FormControl();
  selectedPartner?: Partner;
  pagination?: PartnerPagination;

  __members: MemberModel[] = [];
  __checkedUsers: string[] = [];

  title: string = '';
  btnTitle: string = '';
  memberClassId = '';

  signoutBlockConfigForm!: FormGroup;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _activatedRoute: ActivatedRoute,
    private _partnerService: PartnerService,
    private __memberService: MemberService,
    private __vendorService: VendorService,

    private router: Router,
    private _matDialog: MatDialog
  ) {
    switch (this.router.url) {
      case '/member/partner/all':
        this.title = '전체목록';
        this.btnTitle = '대본등록';
        this.memberClassId = environment.constants.classeIds.mainOffice;
        break;
      case '/member/partner/main-office':
        this.title = '본사목록';
        this.btnTitle = '대본등록';
        this.memberClassId = environment.constants.classeIds.branch;
        break;
      case '/member/partner/branch':
        this.title = '대본목록';
        this.btnTitle = '부본등록';
        this.memberClassId = environment.constants.classeIds.division;
        break;
      case '/member/partner/division':
        this.title = '부본목록';
        this.btnTitle = '총판등록';
        this.memberClassId = environment.constants.classeIds.office;
        break;
      case '/member/partner/office':
        this.title = '총판목록';
        this.btnTitle = '매장등록';
        this.memberClassId = environment.constants.classeIds.store;
        break;
      case '/member/partner/store':
        this.title = '매장목록';
        this.btnTitle = '회원등록';
        this.memberClassId = environment.constants.classeIds.user;
        break;
      default:
        console.log(this.router.url);
        break;
    }
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the pagination
    this._partnerService.pagination$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((pagination: PartnerPagination | undefined) => {
        // Update the pagination
        this.pagination = pagination;

        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products

    this._activatedRoute.data.subscribe((data) => {
      let listMemberResult: ListMembersResponse.Result = data['listPartner'];
      this.listPartner$ = of(listMemberResult.getMembersList());

      let listMemberLevelsResult: ListMemberLevelsResponse.Result =
        data['listMemberLevels'];
      this.listMemberLevels$ = of(listMemberLevelsResult.getMemberLevelsList());

      // Mark for check
      this._changeDetectorRef.markForCheck();
    });

    this.__vendorService
      .listVendors()
      .then((l) => {
        console.log('list', l.getVendorsList());
      })
      .catch((e) => {
        console.log('error', e);
      });

    // Set signout block config
    this.__signoutBlockConfirmConfig();
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {
    if (this._sort && this._paginator) {
      // Set the initial sort
      this._sort.sort({
        id: 'signinId',
        start: 'asc',
        disableClear: true,
      });

      // Mark for check
      this._changeDetectorRef.detectChanges();

      // If the partner changes the sort order...
      this._sort.sortChange
        .pipe(takeUntil(this._unsubscribeAll))
        .subscribe(() => {
          // Reset back to the first page
          this._paginator.pageIndex = 0;
        });

      // Get products if sort or page changes
      merge(this._sort.sortChange, this._paginator.page)
        .pipe(
          switchMap(() => {
            this.isLoading = true;

            let req = new ListMembersRequest();
            let paging = new Pagination();
            paging.setPage(this._paginator.pageIndex);
            paging.setPageSize(this._paginator.pageSize);

            req.setPagination(paging);
            let search = new ListMembersRequest.Search();
            search.setMemberClassId(environment.constants.classeIds.mainOffice);
            req.setSearch(search);

            return of(this.__memberService.listMembers(req));
          }),
          map(() => {
            this.isLoading = false;
          })
        )
        .subscribe();
    }
  }

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  viewUserDetail(id: string): void {
    let url: string = 'member/user/' + id;
    this.router.navigateByUrl(url);
  }

  viewSlot(id: string): void {
    let url: string = 'game/slot';
    this.router.navigateByUrl(url);
  }
  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  __onClickRegist(event: MouseEvent, partner?: MemberModel): void {
    const dialogRef = this._matDialog.open(RegistComposeComponent, {
      data: {
        title: this.btnTitle,
        parent: partner,
        classId: this.memberClassId,
      },
    });

    dialogRef
      .afterClosed()
      .pipe(take(1))
      .subscribe((result: RegistComposeResult) => {
        console.log('Compose dialog was closed: ', result);
      });
  }
  __onClickCash(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CashComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickCommission(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CommissionComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickComp(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CompComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickCoupon(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CouponComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickMessage(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(MessageComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }

  __getRateTooltop(): string {
    const tempRate = 0;
    const resultTooltip = `
    요율확인 \n 카지노-바카라: ${tempRate}% \n 카지노-룰렛: ${tempRate}% \n 카지노-드레곤타이거: ${tempRate}% \n 카지노-그외:${tempRate}% \n 슬롯:${tempRate}% \n 카지노루징: ${tempRate}% \n 슬롯루징: ${tempRate}%
    `;
    return resultTooltip;
  }

  __getStateString(state: any): string {
    console.log(state.getState());

    return '정상';
  }
  __signoutBlockConfirmConfig(): void {
    this.signoutBlockConfigForm = this._formBuilder.group({
      title: '알림',
      message: '로그아웃되었습니다.',
      icon: this._formBuilder.group({
        show: true,
        name: 'heroicons_outline:exclamation',
        color: 'warn',
      }),
      actions: this._formBuilder.group({
        confirm: this._formBuilder.group({
          show: true,
          label: '확인',
          color: 'warn',
        }),
        cancel: this._formBuilder.group({
          show: false,
          label: '취소',
        }),
      }),
      dismissible: true,
    });
  }
  __onClickSignoutBlock(event: MouseEvent): void {
    // Open the dialog and save the reference of it
    const dialogRef = this._fuseConfirmationService.open(
      this.signoutBlockConfigForm.value
    );
  }

  __onChangeChkUsersAll(event: MatCheckboxChange): void {
    let checked = event.checked;
    this.chkUsers.forEach((c, i) => {
      c.checked = checked;
    });
    this.__updateCheckedUsers();
  }

  __onChangeChkUsers(event: MatCheckboxChange): void {
    this.__updateCheckedUsers();
  }

  __updateCheckedUsers(): void {
    this.__checkedUsers = [];
    this.chkUsers.forEach((c, i) => {
      if (c.checked) {
        this.__checkedUsers.push(c.value);
      }
    });
  }

  __onClickChangeLevel(member_level_id: string): void {
    console.log('member_level_id', member_level_id);
  }

  __getMemberState(member: MemberModel): string {
    const state = member.getState();
    let resultStr = '';

    switch (state) {
      case MemberState.NONE:
        resultStr = '없음';
        break;
      case MemberState.NORMAL:
        resultStr = '정상';
        break;
      case MemberState.PENDING:
        resultStr = '대기';
        break;
      case MemberState.WITHDRAWAL:
        resultStr = '탈퇴';
        break;
      case MemberState.DORMANCY:
        resultStr = '휴면';
        break;
      case MemberState.BLACKLIST:
        resultStr = '블랙리스트';
        break;
      case MemberState.SUSPENDED:
        resultStr = '정지';
        break;
      default:
        break;
    }

    return resultStr;
  }
}
